window.Gleam.initSearch([{"doc":"kmh","title":"kmh","content":"# Knuth’s Multiplicative Hash\n\n[![Package Version](https://img.shields.io/hexpm/v/kmh)](https://hex.pm/packages/kmh)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/kmh/)\n\n```sh\ngleam add kmh@1\n```\n```gleam\nimport kmh\n\npub fn main() {\n  // Create a new hasher with a prime, modular inverse and random number\n  let hasher = kmh.new(\n    prime: 1_580_030_173,\n    mod_inverse: 59_260_789,\n    random: 1_163_945_558,\n  )\n  // Ensure hasher parameters are valid. This is optional, but recommended.\n  let assert Ok(hasher) = kmh.validate(hasher)\n\n  // Encode a value. For example, 15 can be a sequential internal ID that we\n  // don’t want to expose.\n  let public_id = kmh.encode(hasher, 15)\n  // -&gt; 1103647397\n\n  // Decode back to the original value.\n  let internal_id = kmh.decode(hasher, 1_103_647_397)\n  // -&gt; 15\n}\n```\n\n&gt; **Important note:**\n&gt; Do not reuse the hasher parameters hereabove in production. This will defeat\n&gt; the purpose of using this library.\n\nFurther documentation can be found at &lt;https://hexdocs.pm/kmh&gt;.\n\n## Targets\n\nErlang only for now. But contributions are welcome!\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n","url":"index.html"},{"doc":"kmh","title":"Error","content":"pub type Error {\n  TooLargePrime\n  TooLargeRandom\n  NotAPrime\n  InvalidModularInverse\n}\n Errors that can occur when validating a hasher.\nTooLargePrime\n\n\nTooLargeRandom\n\n\nNotAPrime\n\n\nInvalidModularInverse\n\n\nSynonyms:\nkmh.Error\nkmh Error","url":"kmh.html#Error"},{"doc":"kmh","title":"Hasher","content":"pub opaque type Hasher(validated)\n A hash function that generates obfuscated values according to Donald Knuth’s\n multiplicative hashing algorithm.\n\n - The hash function is bijective, meaning that it maps a number to a number\n between 0 and the *maximum value*.\n - The hash function is deterministic, meaning that it always generates the\n same output for the same input.\n - The hash function is collision resistant, meaning that it is impossible to\n find two different numbers that map to the same encoded value.\n - The hash function is “one-way”, in the sense that it is infeasible to find\n the number that maps to a given encoded value (without knowing the hash\n parameters).\n\nSynonyms:\nkmh.Hasher\nkmh Hasher","url":"kmh.html#Hasher"},{"doc":"kmh","title":"Unvalidated","content":"pub type Unvalidated\n\n\nSynonyms:\nkmh.Unvalidated\nkmh Unvalidated","url":"kmh.html#Unvalidated"},{"doc":"kmh","title":"Valid","content":"pub type Valid\n\n\nSynonyms:\nkmh.Valid\nkmh Valid","url":"kmh.html#Valid"},{"doc":"kmh","title":"decode","content":"pub fn decode(h: Hasher(Valid), number: Int) -&gt; Int\n Decode a number.\n\n The encoded value must be between 0 and the *maximum value*.\nSynonyms:\nkmh.decode\nkmh decode","url":"kmh.html#decode"},{"doc":"kmh","title":"encode","content":"pub fn encode(h: Hasher(Valid), number: Int) -&gt; Int\n Encode a number.\n\n The number must be between 0 and the *maximum value*.\nSynonyms:\nkmh.encode\nkmh encode","url":"kmh.html#encode"},{"doc":"kmh","title":"max_size","content":"pub fn max_size(\n  h: Hasher(Unvalidated),\n  max_size: Int,\n) -&gt; Hasher(Unvalidated)\n Set the maximum size (in bits) of the values that can be generated.\n\n Defaults to **31** bits, leading to a *maximum value* of **2^31 - 1**.\n\n **Note:** This must be called before `validate`.\nSynonyms:\nkmh.max_size\nkmh max_size","url":"kmh.html#max_size"},{"doc":"kmh","title":"new","content":"pub fn new(\n  prime prime: Int,\n  mod_inverse mod_inverse: Int,\n  random random: Int,\n) -&gt; Hasher(Unvalidated)\n Create a new hasher.\n\n The `prime`, `mod_inverse` and `random` values must be chosen such that the\n following conditions are met:\n\n - The `prime` is a prime number.\n - The `mod_inverse` is the modular multiplicative inverse of the `prime`.\n - The `random` value is a random number between 0 and the *maximum value*.\nSynonyms:\nkmh.new\nkmh new","url":"kmh.html#new"},{"doc":"kmh","title":"unsafe_decode","content":"pub fn unsafe_decode(h: Hasher(a), number: Int) -&gt; Int\n Unsafe version of decode.\n\n This function bypasses the validation of the hasher. Only use this if you\n know what you are doing.\nSynonyms:\nkmh.unsafe_decode\nkmh unsafe_decode","url":"kmh.html#unsafe_decode"},{"doc":"kmh","title":"unsafe_encode","content":"pub fn unsafe_encode(h: Hasher(a), number: Int) -&gt; Int\n Unsafe version of encode.\n\n This function bypasses the validation of the hasher. Only use this if you\n know what you are doing.\nSynonyms:\nkmh.unsafe_encode\nkmh unsafe_encode","url":"kmh.html#unsafe_encode"},{"doc":"kmh","title":"validate","content":"pub fn validate(\n  h: Hasher(Unvalidated),\n) -&gt; Result(Hasher(Valid), Error)\n Validate the hasher.\n\n Returns an error if the hasher was not created with valid parameters.\n Returns the hasher if it was valid.\nSynonyms:\nkmh.validate\nkmh validate","url":"kmh.html#validate"},{"doc":"kmh","title":"kmh","content":"","url":"kmh.html"}]);